unit uMain;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.StdCtrls, Vcl.ComCtrls, Vcl.Themes,
  Vcl.Mask, Vcl.ExtCtrls;

type
  // declared TEdit & TButton as we are going to override the style behaviour on this form
  TEdit = Class (Vcl.StdCtrls.TEdit);
  TButton = Class (Vcl.stdCtrls.TButton);

  // This is the class that will intercept paint messages and mange the style to TEdit
  TEditStyleHookColor = class(TEditStyleHook)
  private
    procedure UpdateColors;
  protected
    procedure WndProc(var Message: TMessage); override;
  public
    constructor Create(AControl: TWinControl); override;
  end;

  TFrmMain = class(TForm)
    Edit1: TEdit;
    Edit2: TEdit;
    Button1: TButton;
    procedure Button1Click(Sender: TObject);
  private
  public
    { Public declarations }
  end;

var
  FrmMain: TFrmMain;

implementation

{$R *.dfm}

uses
  Vcl.Styles;

type
  TWinControlH= class(TWinControl);


constructor TEditStyleHookColor.Create(AControl: TWinControl);
begin
  inherited;
  UpdateColors;
end;

procedure TEditStyleHookColor.UpdateColors;
var
  {
    TCustomStyleServices has information about a style and
    perform painting of custom controls
  }
  CurrentStyle: TCustomStyleServices;
begin
 if Control.Enabled then
 begin
  // Gets its colour information from the Colour property
  Brush.Color := TWinControlH(Control).Color;
  FontColor   := TWinControlH(Control).Font.Color;
 end
 else
 begin
  {
    Call StyleServices to access the methods and properties of
    TCustomStyleServices
  }
  CurrentStyle := StyleServices;
  // Gets its colour from the Style
  Brush.Color := CurrentStyle.GetStyleColor(scEditDisabled);
  FontColor := CurrentStyle.GetStyleFontColor(sfEditBoxTextDisabled);
 end;
end;

procedure TEditStyleHookColor.WndProc(var Message: TMessage);
begin
  // When control colour messages are intercepted, custom paint the TEdit
  case Message.Msg of
    CN_CTLCOLORMSGBOX..CN_CTLCOLORSTATIC:
      begin
        UpdateColors;
        SetTextColor(Message.WParam, ColorToRGB(FontColor));
        SetBkColor(Message.WParam, ColorToRGB(Brush.Color));
        Message.Result := LRESULT(Brush.Handle);
        Handled := True;
      end;
    CM_ENABLEDCHANGED:
      begin
        UpdateColors;
        Handled := False;
      end
  else
    inherited WndProc(Message);
  end;
end;

procedure TFrmMain.Button1Click(Sender: TObject);
begin
 Edit1.Color:=clBlue;
end;


initialization
 TStyleManager.Engine.RegisterStyleHook(TEdit, TEditStyleHookColor);
 // Register a blank StyleHook so that TButton on this form looks like standard styled button
 //TStyleManager.Engine.RegisterStyleHook(TButton, TStyleHook);
end.

